/********************************************************************************
 * Copyright (c) 2024 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CPU_OPERATOR_ARGMAXIMPL_H_
#define AIDGE_CPU_OPERATOR_ARGMAXIMPL_H_

#include <array>
#include <memory>
#include <tuple>
#include <vector>

#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/ArgMax.hpp"
#include "aidge/utils/Registrar.hpp"
#include "aidge/utils/Types.h"

namespace Aidge {
class ArgMaxImplForward_cpu
    : public Registrable<ArgMaxImplForward_cpu,
                        std::tuple<DataType, DataType>,
                        void(std::int32_t,
                            DimSize_t,
                            const std::vector<DimSize_t>&,
                            const void *,
                            void *)> {};
class ArgMaxImplBackward_cpu
    : public Registrable<ArgMaxImplBackward_cpu,
                        std::tuple<DataType, DataType>,
                        void(std::int32_t,
                            DimSize_t,
                            const std::vector<DimSize_t>&,
                            const void *,
                            void *)> {};

class ArgMaxImpl_cpu : public OperatorImpl {
   public:
    ArgMaxImpl_cpu(const ArgMax_Op& op) : OperatorImpl(op, "cpu") {}

    static std::unique_ptr<ArgMaxImpl_cpu> create(const ArgMax_Op &op) {
        return std::make_unique<ArgMaxImpl_cpu>(op);
    }

   public:
    void forward() override;
};

namespace {
static Registrar<ArgMax_Op> registrarArgMaxImpl_cpu("cpu", Aidge::ArgMaxImpl_cpu::create);
}  // namespace
}  // namespace Aidge

#endif /* AIDGE_CPU_OPERATOR_ARGMAXIMPL_H_ */
